﻿using FLY.OBJComponents.Common;
using FLY.Weight.IService;
using FLY.Weight.Server;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace FLY.Weight.UI.Server
{
    /// <summary>
    /// WdSetup.xaml 的交互逻辑
    /// </summary>
    public partial class WdSetup : Window
    {

        public event PropertyChangedEventHandler PropertyChanged;

        WdSetupVm viewModel;

        public WdSetup()
        {
            InitializeComponent();

        }

        public void Init(TDGage gage)
        {
            viewModel = new WdSetupVm();
            viewModel.Init(gage);
            this.DataContext = viewModel;
        }

        private void btnOkClick(object sender, RoutedEventArgs e)
        {
            viewModel.Ok();
        }
        private void btnGenClick(object sender, RoutedEventArgs e)
        {
            viewModel.GenV3();
        }

        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            viewModel.PlcGroupItem2Version();
        }
    }

    public class WdSetupVm : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;
        
        [PropertyChanged.DoNotCheckEquality]
        public int[] BinCnts { get; set; }
        public ObservableCollection<PlcGroupItem> PlcGroupItems { get; private set; } 
        public PlcGroupItem SelectedItem { get; set; }


        public ObservableCollection<DataVersion> DataVersions { get; } = new ObservableCollection<DataVersion>{
            new DataVersion(){Name = "V0", Version =0},
            new DataVersion(){Name = "V1", Version =1},
            new DataVersion(){Name = "V2", Version =2},
            new DataVersion(){Name = "V3", Version =3},
            new DataVersion(){Name = "V4", Version =4},
            new DataVersion(){Name = "V5", Version =5},
            new DataVersion(){Name = "V6", Version =6},
            new DataVersion(){Name = "V7", Version =7}
        };

        public ObservableCollection<WeightStyle> WeightStyles { get; } = new ObservableCollection<WeightStyle>{
            new WeightStyle(){Name = "肥版", ItemStyle =0},
            new WeightStyle(){Name = "长版", ItemStyle = 1}
        };

        public int DataVersion { get; set; }
        public int ItemStyle { get; set; }

        public string[] NumberNames { get; set; }

        public string FileVersion { get; set; }
        TDGage gage;
        bool isSkipPlcGroupItem2Version = false;
        public WdSetupVm()
        {

        }

        public void Init(TDGage gage)
        {

            this.gage = gage;
            Init_PlcGroupItems();
            gage.mData.VersionStringParse(out int version, out int itemStyle);
            DataVersion = version;
            ItemStyle = itemStyle;
            NumberNames = gage.mData.NumberNames;
            FileVersion = gage.mData.FileVersion;

            isSkipPlcGroupItem2Version = true;
        }

        private void Init_PlcGroupItems()
        {

            //查找 脚本/称重 下的全部 设备连接变量表_xxx 的文件夹
            DirectoryInfo directoryInfo = new DirectoryInfo(@"plcgroups");
            if (!directoryInfo.Exists)
            {
                //异常
                return;
            }

            var dirs = directoryInfo.GetDirectories();



            List<PlcGroupItem> plcGroupItems = new List<PlcGroupItem>();
            foreach (var dir in dirs)
            {
                if (dir.Name.StartsWith("设备连接变量表_"))
                {

                    PlcGroupItem item = new PlcGroupItem();
                    item.Name = dir.Name.Substring("设备连接变量表_".Length);
                    //System.IO.Path.Combine(dir.FullName, "Generated", "plcgroup.json");
                    item.Path = $@"plcgroups\{dir.Name}\Generated\plcgroup.json";
                    if (item.Name == gage.mData.FileVersion) { 
                        //就是它

                    }

                    plcGroupItems.Add(item);
                }
            }
            PlcGroupItems = new ObservableCollection<PlcGroupItem>(plcGroupItems);

            //先比较 设备连接变量表_FileVersion 是否一样
            var currPgi =plcGroupItems.Find(pgi => pgi.Name == gage.mData.FileVersion);
            if (currPgi != null) {
                if (IsSameContent(currPgi.Path, "plcgroup.json"))
                {
                    //找到了
                    SelectedItem = currPgi;
                    return;
                }
            }

            foreach (var item in PlcGroupItems)
            {
                if (IsSameContent(item.Path, "plcgroup.json"))
                {
                    //找到了
                    SelectedItem = item;
                    return;
                }
            }
            SelectedItem = null;
        }
        public void PlcGroupItem2Version()
        {
            if (isSkipPlcGroupItem2Version)//解决第1次绑定界面时，SelectedItem 被绑定，从null 变为有值， PlcGroupItem2Version就会触发， 不想这样
            {
                isSkipPlcGroupItem2Version = false;
                return;
            }
            if (SelectedItem == null) return;
            if (string.IsNullOrEmpty(SelectedItem.Name))
                return;
            Regex regex = new Regex(@"(\d+)_v(\d+)");
            var match = regex.Match(SelectedItem.Name);
            if (match.Success)
            {
                DataVersion = int.Parse(match.Groups[2].Value);

                int itemsCnt = match.Groups[1].Value.Count();
                if (itemsCnt <= 3)
                    ItemStyle = 0;
                else
                    ItemStyle = 1;
                FileVersion = match.Groups[0].Value;
            }
        }
        static bool IsSameContent(string filePath1, string filePath2)
        {
            //创建一个哈希算法对象
            using (HashAlgorithm hash = HashAlgorithm.Create())
            {
                using (FileStream file1 = new FileStream(filePath1, FileMode.Open), file2 = new FileStream(filePath2, FileMode.Open))
                {
                    byte[] hashByte1 = hash.ComputeHash(file1);//哈希算法根据文本得到哈希码的字节数组
                    byte[] hashByte2 = hash.ComputeHash(file2);
                    return Enumerable.SequenceEqual(hashByte1, hashByte2);

                    //string str1 = BitConverter.ToString(hashByte1);//将字节数组装换为字符串
                    //string str2 = BitConverter.ToString(hashByte2);
                    //return (str1 == str2);//比较哈希码
                }
            }
        }
        public void Ok()
        {
            var item = SelectedItem;
            if (item == null)
            {
                MessageBox.Show("请选择型号！！！");
                return;
            }

            if (MessageBox.Show("需要重启，才能生效", "提示", MessageBoxButton.OKCancel) == MessageBoxResult.OK)
            {
                File.Copy(item.Path, "plcgroup.json", true);

                gage.mData.NumberNames = NumberNames;
                gage.mData.Version = gage.mData.CreateVersionString(DataVersion, ItemStyle);
                gage.mData.FileVersion = FileVersion;
                gage.mData.SaveNumberNames();


                //切换为exe的目录，这样程序再次进入时，才能 进入Gage1 文件夹
                System.Environment.CurrentDirectory = "../";
                //重启软件
                AppHelper.AppJustOne.Restart();
            }
        }


        public bool GenV3()
        {
            if (BinCnts == null || BinCnts.Count() == 0) {
                MessageBox.Show("输入格式出错", "异常", MessageBoxButton.OK, MessageBoxImage.Error);
                return false;
            }
            if (BinCnts.Count() > 7) {
                MessageBox.Show($"输入层数={BinCnts.Count()} 大于7层", "异常", MessageBoxButton.OK, MessageBoxImage.Error);
                return false;
            }
            for (int i = 0; i < BinCnts.Count(); i++) {
                if (BinCnts[i] > 6) {
                    MessageBox.Show($"输入第{i+1}层仓数={BinCnts[i]}>6", "异常", MessageBoxButton.OK, MessageBoxImage.Error);
                    return false;
                }
                else if (BinCnts[i] < 2)
                {
                    MessageBox.Show($"输入第{i + 1}层仓数={BinCnts[i]}<2", "异常", MessageBoxButton.OK, MessageBoxImage.Error);
                    return false;
                }
            }
            BinCntsConverter converter = new BinCntsConverter();
            string binCnts_str = (string)converter.Convert(BinCnts, typeof(string), null, null);

            //检测是否已经存在
            string plcGroupName = $"{binCnts_str}_v3";
            string targetPath = $@"plcgroups\设备连接变量表_{plcGroupName}\Generated\plcgroup.json";
            string targetDir = System.IO.Path.GetDirectoryName(targetPath);
            try
            {
                SelectedItem = PlcGroupItems.First(i => i.Name == plcGroupName);
                MessageBox.Show($"{targetPath} 已经存在, 可以在列表选择使用", "通知", MessageBoxButton.OK, MessageBoxImage.Information);
                return true;
            }
            catch { 
                //不存在！！！
            }



            //格式正常了
            //检测 plcgroups/设备连接变量表_6443236_v3/Generated/plcgroup.json 
            string filePath = @"plcgroups\设备连接变量表_6666666_v3\Generated\plcgroup.json";

            if (!File.Exists(filePath)) {
                MessageBox.Show($"{filePath} 不存在,无法生成", "异常", MessageBoxButton.OK, MessageBoxImage.Error);
                return false;
            }

            string json = File.ReadAllText(filePath);
            var  plcgroup = Newtonsoft.Json.JsonConvert.DeserializeObject<PLCGroup>(json);

            //删除多余的设备
            plcgroup.Devices.RemoveRange(BinCnts.Count(), plcgroup.Devices.Count() - BinCnts.Count());
            plcgroup.Variables.RemoveAll(var => var.DeviceIndex >= plcgroup.Devices.Count());
            
            Regex r = new Regex(@"Items\[([0-9])\]");
            Regex r2 = new Regex(@"(?:\w+)_([1-9])");
            //删除每层的仓
            plcgroup.Variables.RemoveAll(var =>
            {
                Match m = r.Match(var.OwnerName);
                if (m.Success)
                {
                    int weighter_idx = int.Parse(m.Groups[1].Value);
                    if (weighter_idx >= BinCnts.Count())
                    {
                        //这个层不应该有，删除
                        return true;
                    }
                    Match m2 = r2.Match(var.PropertyName);
                    if (m2.Success)
                    {
                        int bincnt = int.Parse(m2.Groups[1].Value);

                        if (bincnt > BinCnts[weighter_idx])
                        {
                            //这个仓也不应该有，删除
                            return true;
                        }
                    }
                }
                return false;
            });

            //输出文件
            json = Newtonsoft.Json.JsonConvert.SerializeObject(plcgroup, Newtonsoft.Json.Formatting.Indented);

            //创建文件夹
            if(!Directory.Exists(targetDir))
                Directory.CreateDirectory(targetDir);
            File.WriteAllText(targetPath, json);

            //把这个添加到 PlcGroupItems 
            PlcGroupItem item = new PlcGroupItem()
            {
                Name = plcGroupName,
                Path = targetPath
            };
            PlcGroupItems.Add(item);
            SelectedItem = item;

            MessageBox.Show($"{targetPath} 添加成功, 可以在列表选择使用", "通知", MessageBoxButton.OK, MessageBoxImage.Information);
            return true;
        }
    }
    public class PlcGroupItem
    {
        /// <summary>
        /// 名称
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// 路径
        /// </summary>
        public string Path { get; set; }
    }

    public class DataVersion
    {
        /// <summary>
        /// 名称
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// 版本
        /// </summary>
        public int Version { get; set; }
    }
    public class WeightStyle
    {
        /// <summary>
        /// 名称
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// 版本
        /// </summary>
        public int ItemStyle { get; set; }
    }
    public class BinCntsConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            int[] binCnts = value as int[];
            if (binCnts == null || binCnts.Count() == 0)
                return null;

            string str = "";
            for (int i = 0; i < binCnts.Count(); i++) {
                str += binCnts[i];
            }
            return str;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            string str = value as string;
            if(string.IsNullOrEmpty(str))
                return null;
            List<int> binCnts = new List<int>();
            str = str.Trim();
            for (int i = 0; i < str.Length; i++) {
                
                if (int.TryParse(str[i].ToString(), out int binCnt)) {
                    binCnts.Add(binCnt);
                }
            }
            return binCnts.ToArray();
        }
    }

    public class NumberNamesConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            string[] names = value as string[];
            if (names == null || names.Count() == 0)
                return null;

            string str = "";
            for (int i = 0; i < names.Count(); i++)
            {
                if (i > 0)
                    str += ",";
                str += names[i];

            }
            return str;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            string str = value as string;
            if (string.IsNullOrEmpty(str))
                return null;
            List<string> names = new List<string>();
            var ss = str.Split(',', '，');
            for (int i = 0; i < ss.Length; i++)
            {
                ss[i] = ss[i].Trim();
                if (!string.IsNullOrEmpty(ss[i]))
                    names.Add(ss[i]);
            }
            return names.ToArray();
        }
    }
}
