﻿using FLY.OBJComponents.IService;
using FLY.Weight.Client;
using FLY.Weight.Common;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace FLY.Weight.IService
{
    public interface IWeightSystemService: INotifyPropertyChanged
    {
        /// <summary>
        /// 称重其它附件
        /// </summary>
        WeighterAccessory Accessory { get; }
        /// <summary>
        /// 各层称重
        /// </summary>
        ObservableCollection<WeighterC> Items { get; }

        /// <summary>
        /// PLC代理系统
        /// </summary>
        IPLCProxySystemService PLCos { get; }


        /// <summary>
        /// 层数
        /// </summary>
        int ItemsCnt { get; }

        /// <summary>
        /// 每层仓数
        /// </summary>
        int[] BinCnts { get; }

        /// <summary>
        /// 每层名称
        /// </summary>
        string[] NumberNames { get; }

        /// <summary>
        /// 版本
        /// </summary>
        string Version { get; }

        /// <summary>
        /// 螺杆频率调节增量， 用于 +,- 号
        /// </summary>
        double ScrewMotorFreqInc { get; set; }
    }

    public static class WeightSystemExt 
    {
        public static void VersionStringParse(this IWeightSystemService weightSystem, out int version, out int itemStyle) {

            var versionString = weightSystem.Version;
            if (string.IsNullOrEmpty(versionString))
            {
                version = 3;
                itemStyle = 0;
                return;
            }    

            Regex regex_version = new Regex(@"version=(\d+);?");
            var match_version = regex_version.Match(versionString);
            version = match_version.Success ? int.Parse(match_version.Groups[1].Value) : 3;

            Regex regex_itemStyle = new Regex(@"itemStyle=(\d+);?");
            var match_itemStyle = regex_itemStyle.Match(versionString);
            itemStyle = match_itemStyle.Success ? int.Parse(match_itemStyle.Groups[1].Value) : 0;
        }
        public static string CreateVersionString(this IWeightSystemService weightSystem, int version, int itemStyle)
        {
            return $"version={version};itemStyle={itemStyle}";
        }
    }

}
